unit FindUnit;
{
Copyright (C) 2000 Patrick Chevalley

http://www.astrosurf.org/astropc
pchev@geocities.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
}

interface
Uses SysUtils,SkylibCat,NGCUnit,GCVUnit,GSCUnit,GSCFits,GSCC,BSCUnit,PGCunit,SACunit;

Procedure FindNumNGC(id:Integer ;var ar,de:double; var ok:boolean); stdcall;
Procedure FindNumIC(id:Integer ;var ar,de:double; var ok:boolean); stdcall;
Procedure FindNumMessier(id:Integer ;var ar,de:double; var ok:boolean); stdcall;
Procedure FindNumGCVS(id:shortstring ;var ar,de:double; var ok:boolean); stdcall;
Procedure FindNumGSC(id : ShortString ;var ar,de:double; var ok:boolean); stdcall;
Procedure FindNumGSCF(id : ShortString ;var ar,de:double; var ok:boolean); stdcall;
Procedure FindNumGSCC(id : ShortString ;var ar,de:double; var ok:boolean); stdcall;
Procedure FindNumGC(id:Integer ;var ar,de:double; var ok:boolean); stdcall;
Procedure FindNumHR(id:Integer ;var ar,de:double; var ok:boolean); stdcall;
Procedure FindNumBayer(id:Shortstring ;var ar,de:double; var ok:boolean); stdcall;
Procedure FindNumFlam(id:Shortstring ;var ar,de:double; var ok:boolean); stdcall;
Procedure FindNumHD(id:Integer ;var ar,de:double; var ok:boolean); stdcall;
Procedure FindNumSAO(id:Integer ;var ar,de:double; var ok:boolean); stdcall;
Procedure FindNumBD(id:ShortString ;var ar,de:double; var ok:boolean); stdcall;
Procedure FindNumCD(id:ShortString ;var ar,de:double; var ok:boolean); stdcall;
Procedure FindNumCPD(id:ShortString ;var ar,de:double; var ok:boolean); stdcall;
Procedure FindNumPGC(id:Integer ;var ar,de:double; var ok:boolean); stdcall;
Procedure FindNumSAC(id:ShortString ;var ar,de:double; var ok:boolean); stdcall;

implementation

const blank='               ';

Procedure FindIdxStr(idx, id : string; var ar,de:double; var ok:boolean);
Type idxfil = record
              num : array[1..12] of char;
              ar,de :single;
              end;
var
    imin,imax,i : integer;
    pnum : string;
    fx : file of idxfil ;
    lin : idxfil;
begin
ok:=false;
if not fileexists(idx) then begin
   ErrMsg('Required file : '+idx+'  is not installed. Please update your installation to search using this identifier');
   exit;
end;
AssignFile(fx,idx);
FileMode:=0;
Reset(fx);
imin:=0;
imax := filesize(fx);
repeat
  pnum:=lin.num;
  i:=imin + ((imax-imin) div 2);
  seek(fx,i);
  read(fx,lin);
  if lin.num>id then imax:=i
                else imin:=i;
  if lin.num=id then ok:=true;
until ok or (pnum=lin.num);
CloseFile(fx);
if ok then begin
   ar:=lin.ar/15;
   de:=lin.de;
end;
end;

Procedure FindIdx(idx : string; id:Integer ;var ar,de:double; var ok:boolean);
type idxfil = record num :longint ;
                 ar,de :single;
                 end;
var
    imin,imax,i,pnum : integer;
    fx : file of idxfil ;
    lin : idxfil;
begin
ok:=false;
if not fileexists(idx) then begin
   ErrMsg('Required file : '+idx+'  is not installed. Please update your installation to search using this identifier');
   exit;
end;
AssignFile(fx,idx);
FileMode:=0;
Reset(fx);
imin:=0;
imax := filesize(fx);
repeat
  pnum:=lin.num;
  i:=imin + ((imax-imin) div 2);
  seek(fx,i);
  read(fx,lin);
  if lin.num>id then imax:=i
                else imin:=i;
  if lin.num=id then ok:=true;
until ok or (pnum=lin.num);
CloseFile(fx);
if ok then begin
   ar:=lin.ar/15;
   de:=lin.de;
end;
end;

Procedure FindNumNGC(id:Integer ;var ar,de:double; var ok:boolean);
begin
FindIdx(NGCpath+'\ngc.idx',id,ar,de,ok);
end;

Procedure FindNumIC(id:Integer ;var ar,de:double; var ok:boolean);
begin
FindIdx(NGCpath+'\ic.idx',id,ar,de,ok);
end;

Procedure FindNumMessier(id:Integer ;var ar,de:double; var ok:boolean);
begin
FindIdx(NGCpath+'\messier.idx',id,ar,de,ok);
end;

function IsNumber(n : string) : boolean;
var i : integer;
    dummy_double : double;
begin
val(n,Dummy_double,i);
result:= (i=0) ;
end;

Procedure FindNumGCVS(id:shortstring ;var ar,de:double; var ok:boolean);
var buf,buf1,buf2 : string;
begin
buf1:=words(id,'',1,1);
buf2:=words(id,'',2,1);
if buf1<>'u' then buf1:=uppercase(buf1)+blank
             else buf1:=buf1+blank;
if trim(buf1)='NSV' then buf2:=padzeros(buf2,5);
if copy(buf1,1,1)='V' then begin
   buf:=trim(copy(buf1,2,99));
   if IsNumber(buf) then buf1:='V'+padzeros(buf,4)+blank;
end;
buf2:=uppercase(buf2)+blank;
if copy(buf2,1,1)='V' then begin
   buf:=trim(copy(buf2,2,99));
   if IsNumber(buf) then buf2:='V'+padzeros(buf,4)+blank;
end;
id:=copy(buf1,1,6)+copy(buf2,1,6);
FindIdxStr(GCVpath+'\gcvs.idx',id,ar,de,ok);
end;

Procedure FindNumGSC(id : ShortString ;var ar,de:double; var ok:boolean);
var smnum,num,p : integer;
begin
ok:=false;
id:=trim(id);
p:=pos(' ',id);
if p>0 then begin
   smnum:=strtoint(copy(id,1,p-1));
   num:=strtoint(copy(id,p+1,5));
   FindGSCnum(smnum,num,ar,de,ok);
end;
end;

Procedure FindNumGSCF(id : ShortString ;var ar,de:double; var ok:boolean);
var smnum,num,p : integer;
begin
ok:=false;
id:=trim(id);
p:=pos(' ',id);
if p>0 then begin
   smnum:=strtoint(copy(id,1,p-1));
   num:=strtoint(copy(id,p+1,5));
   FindGSCFnum(smnum,num,ar,de,ok);
end;
end;

Procedure FindNumGSCC(id : ShortString ;var ar,de:double; var ok:boolean);
var smnum,num : integer;
    p : integer;
begin
ok:=false;
id:=trim(id);
p:=pos(' ',id);
if p>0 then begin
   smnum:=strtoint(copy(id,1,p-1));
   num:=strtoint(copy(id,p+1,5));
   FindGSCCnum(smnum,num,ar,de,ok);
end;
end;

Procedure FindNumGC(id:Integer ;var ar,de:double; var ok:boolean);
begin
FindIdx(BSCpath+'\gc.idx',id,ar,de,ok);
end;

Procedure FindNumHR(id:Integer ;var ar,de:double; var ok:boolean);
var x1,x2,y1,y2 : double;
    bscrok : boolean;
    lin : bscrec;
begin
x1 := 0;
x2 := 24;
y1 := -89.9997;
y2 := 89.9997;
OpenBSC(x1,x2,y1,y2,BSCrok);
if not BSCrok then begin ok:=false; exit; end;
ok := false;
repeat
  ReadBSC(lin,BSCrok);
  if not BSCrok then break;
  if lin.bs=id then ok:=true;
until ok ;
CloseBSC;
ar:=lin.ar/100000/15;
de:=lin.de/100000;
end;

Procedure FindNumBayer(id:Shortstring ;var ar,de:double; var ok:boolean);
var x1,x2,y1,y2 : double;
    bscrok : boolean;
    lin : bscrec;
    cons,bayer : string;
begin
bayer:=lowercase(trim(copy(id,1,pos(' ',id))));
cons:=lowercase(trim(copy(id,pos(' ',id),99)));
x1 := 0;
x2 := 24;
y1 := -89.9997;
y2 := 89.9997;
OpenBSC(x1,x2,y1,y2,BSCrok);
if not BSCrok then begin ok:=false; exit; end;
ok := false;
repeat
  ReadBSC(lin,BSCrok);
  if not BSCrok then break;
  if (lowercase(trim(lin.cons))=cons)and(lowercase(trim(copy(lin.bayer,1,length(bayer))))=bayer) then ok:=true;
until ok ;
CloseBSC;
ar:=lin.ar/100000/15;
de:=lin.de/100000;
end;

Procedure FindNumFlam(id:Shortstring ;var ar,de:double; var ok:boolean);
var x1,x2,y1,y2 : double;
    bscrok : boolean;
    lin : bscrec;
    cons : string;
    flam : byte;
begin
flam:=strtoint(trim(copy(id,1,pos(' ',id))));
cons:=lowercase(trim(copy(id,pos(' ',id),99)));
x1 := 0;
x2 := 24;
y1 := -89.9997;
y2 := 89.9997;
OpenBSC(x1,x2,y1,y2,BSCrok);
if not BSCrok then begin ok:=false; exit; end;
ok := false;
repeat
  ReadBSC(lin,BSCrok);
  if not BSCrok then break;
  if (lowercase(trim(lin.cons))=cons)and(lin.flam=flam) then ok:=true;
until ok ;
CloseBSC;
ar:=lin.ar/100000/15;
de:=lin.de/100000;
end;

Procedure FindNumHD(id:Integer ;var ar,de:double; var ok:boolean);
begin
FindIdx(BSCpath+'\hd.idx',id,ar,de,ok);
end;

Procedure FindNumSAO(id:Integer ;var ar,de:double; var ok:boolean);
begin
FindIdx(BSCpath+'\sao.idx',id,ar,de,ok);
end;

Procedure FindNumBD(id:ShortString ;var ar,de:double; var ok:boolean);
var sign : string;
    zone,num,p : integer;
begin
id:=trim(id);
p:=pos(' ',id);
if p>0 then begin
   sign:=copy(id,1,1);
   zone:=strtoint(sign+trim(copy(id,2,p-1)));
   num:=strtoint(sign+trim(copy(id,p+1,5)));
   num:=100000*zone + num;
   FindIdx(BSCpath+'\bd.idx',num,ar,de,ok);
end;
end;

Procedure FindNumCD(id:ShortString ;var ar,de:double; var ok:boolean);
var sign : string;
    zone,num,p : integer;
begin
id:=trim(id);
p:=pos(' ',id);
if p>0 then begin
   sign:=copy(id,1,1);
   zone:=strtoint(sign+trim(copy(id,2,p-1)));
   num:=strtoint(sign+trim(copy(id,p+1,5)));
   num:=100000*zone + num;
   FindIdx(BSCpath+'\cd.idx',num,ar,de,ok);
end;
end;

Procedure FindNumCPD(id:ShortString ;var ar,de:double; var ok:boolean);
var sign : string;
    zone,num,p : integer;
begin
id:=trim(id);
p:=pos(' ',id);
if p>0 then begin
   sign:=copy(id,1,1);
   zone:=strtoint(sign+trim(copy(id,2,p-1)));
   num:=strtoint(sign+trim(copy(id,p+1,5)));
   num:=100000*zone + num;
   FindIdx(BSCpath+'\cp.idx',num,ar,de,ok);
end;
end;

Procedure FindNumPGC(id:Integer ;var ar,de:double; var ok:boolean);
begin
FindIdx(PGCpath+'\pgc.idx',id,ar,de,ok);
end;

Procedure FindNumSAC(id : ShortString ;var ar,de:double; var ok:boolean); stdcall;
Type idxfil = record
              num : string[18];
              ar,de :single;
              end;
var
    imin,imax,i : integer;
    pnum,idx : string;
    fx : file of idxfil ;
    lin : idxfil;
begin
id:=Uppercase(stringreplace(id,' ','',[rfReplaceAll]));
idx:=SACpath+'\sac.idx';
ok:=false;
if not fileexists(idx) then begin
   ErrMsg('Required file : '+idx+'  is not installed. Please update your installation to search using this identifier');
   exit;
end;
AssignFile(fx,idx);
FileMode:=0;
Reset(fx);
imin:=0;
imax := filesize(fx);
repeat
  pnum:=lin.num;
  i:=imin + ((imax-imin) div 2);
  seek(fx,i);
  read(fx,lin);
  if lin.num>id then imax:=i
                else imin:=i;
  if lin.num=id then ok:=true;
until ok or (pnum=lin.num);
CloseFile(fx);
if ok then begin
   ar:=lin.ar/15;
   de:=lin.de;
end;
end;

end.
